SystemOrganization addCategory: #'Zodiac-Core'!

Object subclass: #ZdcAbstractSSLSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcAbstractSSLSession commentStamp: 'SvenVanCaekenberghe 5/18/2011 10:05' prior: 0!
I am ZdcAbstractSSLSession, an object managing the secure communication between two parties.

I define the abstract interface for my subclasses.

More specifically, I handle connection setup handshaking as well as the encryption
and decryption of data travelling between two parties.

Apart from instanciating and later explicitely destroying me, I am used by feeding data 
into me using the methods in my operations protocol. These might result in data that
has to be sent to the other side.

I am propably too primitive to be used directly, see ZdcSecureSocketStream for a higher level client.!

!ZdcAbstractSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:07'!
accept: srcBuf from: start to: stop into: dstBuf
	"Start or continue the server handshake using the given input token"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:08'!
certificateName
	"The name of the local certificate to provide to the remote peer"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:08'!
certificateName: aString
	"Sets the name of the local certificate to provide to the remote peer.
	OpenSSL: The name is the full path to a .pem file.
	WinSSL: The name is matched against the 'subject' of a certificate in the cert store"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:08'!
certificateVerificationState
	"Returns the certificate verification bits. The returned value indicates
	whether the certificate is valid. The two standard values are:
		0   - The certificate is valid
		-1  - No certificate has been provided by the peer
	Otherwise, the result is a bit mask of the following values:
		1   - If set, there is an unspecified issue with the cert (generic error)
		2   - If set, the root CA is untrusted (usually a self-signed cert)
		4   - If set, the certificate is expired
		8   - If set, the certificate is used for the wrong purpose
		16 - If set, the CN of the certificate is invalid
		32 - If set, the certificate was revoked"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:08'!
connect: srcBuf from: start to: stop into: dstBuf
	"Start or continue the client handshake using the given input token"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:08'!
decrypt: srcBuf from: start to: stop into: dstBuf
	"Decrypt the input in srcBuf from start to stop into dstBuf."

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 5/18/2011 10:09'!
destroy
	"Do any necessary cleanup when I am no longer needed"
	
	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:09'!
encrypt: srcBuf from: start to: stop into: dstBuf
	"Encrypt the input in srcBuf from start to stop into dstBuf."

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:09'!
isConnected
	"Returns true if the SSL handshake has been completed"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:10'!
peerCertificateName
	"Returns the certificate name of the remote peer.
	The method only returns a name if the certificate has been verified"

	self subclassResponsibility! !

!ZdcAbstractSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:10'!
sslState
	"Returns the current state of the SSL connection:
		0 - Unused
		1 - In accept handshake
		2 - In connect handshake
		3 - Connected"
		
	self subclassResponsibility! !

ZdcAbstractSSLSession subclass: #ZdcPluginSSLSession
	instanceVariableNames: 'handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcPluginSSLSession commentStamp: 'SvenVanCaekenberghe 9/13/2011 09:56' prior: 0!
I am ZdcPluginSSLSession, an object managing the secure communication between two parties.

I am a ZdcAbstractSSLSession.

I am a wrapper for the SqueakSSL plugin.

I am probably too primitive to be used directly, see ZnSecureSocketStream for a higher level client.

Ackowledgement: based on the original SqueakSSL code.!

!ZdcPluginSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
accept: srcBuf from: start to: stop into: dstBuf
	"Start or continue the server handshake using the given input token"

	^ self primitiveSSL: handle accept: srcBuf startingAt: start count: stop - start + 1 into: dstBuf! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
certificateName
	"The name of the local certificate to provide to the remote peer"

	^ self primitiveSSL: handle getStringProperty: 1! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
certificateName: aString
	"Sets the name of the local certificate to provide to the remote peer.
	OpenSSL: The name is the full path to a .pem file.
	WinSSL: The name is matched against the 'subject' of a certificate in the cert store"

	^ self primitiveSSL: handle setStringProperty: 1 toValue: (aString ifNil: [ ' ' ])! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
certificateVerificationState
	"Returns the certificate verification bits. The returned value indicates
	whether the certificate is valid. The two standard values are:
		0   - The certificate is valid
		-1  - No certificate has been provided by the peer
	Otherwise, the result is a bit mask of the following values:
		1   - If set, there is an unspecified issue with the cert (generic error)
		2   - If set, the root CA is untrusted (usually a self-signed cert)
		4   - If set, the certificate is expired
		8   - If set, the certificate is used for the wrong purpose
		16 - If set, the CN of the certificate is invalid
		32 - If set, the certificate was revoked"
		
	^ self primitiveSSL: handle getIntProperty: 3! !

!ZdcPluginSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
connect: srcBuf from: start to: stop into: dstBuf
	"Start or continue the client handshake using the given input token"

	^ self primitiveSSL: handle connect: srcBuf startingAt: start count: stop - start + 1 into: dstBuf! !

!ZdcPluginSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 12/6/2011 11:32'!
decrypt: srcBuf from: start to: stop into: dstBuf
	"Decrypt the input in srcBuf from start to stop into dstBuf."

	^ self primitiveSSL: handle decrypt: srcBuf startingAt: start count: stop - start + 1 into: dstBuf! !

!ZdcPluginSSLSession methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
destroy
	"Destroys the platform handle in the VM plugin"

	handle isNil ifTrue: [ ^ self ].  
	self primitiveSSLDestroy: handle.
	handle := nil! !

!ZdcPluginSSLSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 12/6/2011 11:32'!
encrypt: srcBuf from: start to: stop into: dstBuf
	"Encrypt the input in srcBuf from start to stop into dstBuf."

	^ self primitiveSSL: handle encrypt: srcBuf startingAt: start count: stop - start + 1 into: dstBuf! !

!ZdcPluginSSLSession methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
initialize
	"Initialize the receiver"

	handle := self primitiveSSLCreate
! !

!ZdcPluginSSLSession methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
isConnected
	"Returns true if the SSL handshake has been completed"

	^ self sslState = 3! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
peerCertificateName
	"Returns the certificate name of the remote peer.
	The method only returns a name if the certificate has been verified"

	^ self primitiveSSL: handle getStringProperty: 0! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
pluginVersion
	"Returns the version of the plugin"

	^ self primitiveSSL: handle getIntProperty: 0! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle accept: srcbuf startingAt: start count: length into: dstbuf
	"Primitive. Starts or continues a server handshake using the provided data.
	Will eventually produce output to be sent to the client.
	Returns:
		> 0   Number of bytes to be sent to the server
		= 0   Success. The connection is established
		= -1  More input is required
		< -1  Other errors"
		
	<primitive: 'primitiveAccept' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle connect: srcbuf startingAt: start count: length into: dstbuf
	"Primitive. Starts or continues a client handshake using the provided data.
	Will eventually produce output to be sent to the server.
	Returns:
		> 0   Number of bytes to be sent to the server
		= 0   Success. The connection is established
		= -1  More input is required
		< -1  Other errors"
	
	<primitive: 'primitiveConnect' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle decrypt: srcbuf startingAt: start count: length into: dstbuf
	"Primitive. Takes incoming data for decryption and continues to decrypt data.
	Returns the number of bytes produced in the output"

	<primitive: 'primitiveDecrypt' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle encrypt: srcbuf startingAt: start count: length into: dstbuf
	"Primitive. Encrypts the incoming buffer into the result buffer.
	Returns the number of bytes produced as a result"

	<primitive: 'primitiveEncrypt' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle getIntProperty: propID
	"Primitive. Returns a string property from an SSL session"

	<primitive: 'primitiveGetIntProperty' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle getStringProperty: propID
	"Primitive. Returns a string property from an SSL session"

	<primitive: 'primitiveGetStringProperty' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle setIntProperty: propID toValue: anInteger
	"Primitive. Sets a string property in an SSL session"

	<primitive: 'primitiveSetStringProperty' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSL: sslHandle setStringProperty: propID toValue: aString
	"Primitive. Sets a string property in an SSL session"

	<primitive: 'primitiveSetStringProperty' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSLCreate
	"Primitive. Creates and returns a new SSL handle in the VM plugin"

	<primitive: 'primitiveCreate' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'primitives' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
primitiveSSLDestroy: sslHandle
	"Primitive. Destroys the SSL session handle in the VM plugin"

	<primitive: 'primitiveDestroy' module: 'SqueakSSL'>
	
	^ self primitiveFailed! !

!ZdcPluginSSLSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 10:01'!
sslState
	"Returns the current state of the SSL connection:
		0 - Unused
		1 - In accept handshake
		2 - In connect handshake
		3 - Connected"
		
	^ self primitiveSSL: handle getIntProperty: 2
		! !

Object subclass: #ZdcIOBuffer
	instanceVariableNames: 'buffer readPointer writePointer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcIOBuffer commentStamp: 'SvenVanCaekenberghe 5/17/2011 18:47' prior: 0!
I am ZdcIOBuffer.

I manage a fixed SequenceableCollection buffer for simultaneous input and output.
I maintain a readPointer and a writePointer.

When data is written to me, it is stored at the end, past my writePointer.
When data is read from me, it is retrieved from the front, past my readPointer.

Invariant: readPointer <= writePointer

My valid contents for reading is defined from contentsStart to contentsEnd, from readPointer + 1 to writePointer.
Data can be added to the free space defined from freeSpaceStart to freeSpaceEnd, from writePointer + 1 to the buffer's' size.

There can be a gap at my front. Compacting moves data if necessary to make (more) room at the end.!

!ZdcIOBuffer class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
on: sequenceableCollection
	^ (self new)
		on: sequenceableCollection;
		yourself! !

!ZdcIOBuffer class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
onByteArrayOfSize: size
	^ self on: (ByteArray new: size)! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
advanceReadPointer: count
	"Advance the read pointer as if count elements were read"
	
	self assert: count >= 0.
	count <= self availableForReading
		ifTrue: [  
			readPointer := readPointer + count ]
		ifFalse: [
			self bufferEmptyError ]  ! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
advanceWritePointer: count
	"Advance the write pointer as if count elements were written"
	
	self assert: count >= 0.
	count <= self availableForWriting 
		ifTrue: [  
			writePointer := writePointer + count ]
		ifFalse: [
			self bufferFullError ]  ! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
availableForReading
	"How many elements can be read ?"

	^ writePointer - readPointer! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
availableForWriting
	"How many elements can be written at the end ?"
	
	^ buffer size - writePointer! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
buffer
	"Return our underlying buffer"
	
	^ buffer! !

!ZdcIOBuffer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
buffer: aSequenceableCollection
	buffer := aSequenceableCollection! !

!ZdcIOBuffer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
bufferEmptyError
	^ self error: 'Buffer empty'! !

!ZdcIOBuffer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
bufferFullError
	^ self error: 'Buffer full'! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
bufferSize
	"Return the raw size of the underlying buffer"
	
	^ buffer size! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
capacity
	"How much room is there both at the front and at the end ?"
	
	^ self gapAtFront + self availableForWriting ! !

!ZdcIOBuffer methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
compact
	"Move data to the front of the buffer to make more room for writing at the end"
	
	| count |
	count := self availableForReading.
	count isZero ifTrue: [ ^ self reset ].
	readPointer isZero ifTrue: [ ^ self ].
	buffer replaceFrom: 1 to: count with: buffer startingAt: readPointer + 1.
	readPointer := 0.
	writePointer := count! !

!ZdcIOBuffer methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
contents
	"Return our readable contents"
	
	^ self isEmpty
		ifTrue: [ buffer copyEmpty ]
		ifFalse: [ buffer copyFrom: self contentsStart to: self contentsEnd ]! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
contentsEnd
	"Return the current valid contents end index into the buffer (inclusive)"
	
	self isEmpty ifTrue: [ self bufferEmptyError ].
	^ writePointer! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
contentsStart
	"Return the current valid contents start index into the buffer (inclusive)"
	
	self isEmpty ifTrue: [ self bufferEmptyError ].
	^ readPointer + 1! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
freeSpaceEnd
	"Return the current valid free space end index into the buffer (inclusive)"
	
	self isFull ifTrue: [ self bufferFullError ]. 
	^ buffer size! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
freeSpaceStart
	"Return the current valid free space start index into the buffer (inclusive)"
	
	self isFull ifTrue: [ self bufferFullError ]. 
	^ writePointer + 1! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
gapAtFront
	"How many elements fit in the gap at the front ?"
	
	^ readPointer! !

!ZdcIOBuffer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
isEmpty
	"Can something be read ?"
	
	^ self availableForReading isZero! !

!ZdcIOBuffer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
isFull
	"Is there room for writing ?"
	
	^ self availableForWriting isZero! !

!ZdcIOBuffer methodsFor: 'reading' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
next
	"Access the next readable element, fail is empty"

	^ self availableForReading > 0
		ifTrue: [
			readPointer := readPointer + 1.
			buffer at: readPointer ]
		ifFalse: [
			self bufferEmptyError ]! !

!ZdcIOBuffer methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
next: count putAll: collection startingAt: offset
	"Add count elements from collection starting at offset. Fail if there is not enough room"
	
	| writeOffset |
	count > self availableForWriting ifTrue: [ self bufferFullError ].  
	writeOffset := self freeSpaceStart.
	buffer replaceFrom: writeOffset to: writeOffset + count - 1 with: collection startingAt: offset.
	self advanceWritePointer: count! !

!ZdcIOBuffer methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
nextPut: anObject
	"Add another element. Fail if full"
	
	self availableForWriting > 0
		ifTrue: [
			writePointer := writePointer + 1 .
			buffer at: writePointer put: anObject ]
		ifFalse: [
			self bufferFullError ].
	^ anObject
! !

!ZdcIOBuffer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
on: sequenceableCollection
	buffer := sequenceableCollection.
	self reset! !

!ZdcIOBuffer methodsFor: 'reading' stamp: 'SvenVanCaekenberghe 5/19/2011 12:50'!
peek
	"Peek the next readable element, fail is empty"

	^ self availableForReading > 0
		ifTrue: [
			buffer at: readPointer + 1 ]
		ifFalse: [
			self bufferEmptyError ]! !

!ZdcIOBuffer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self gapAtFront > 0 
		ifTrue: [ stream nextPut: $-; print: self gapAtFront; space ].
	stream print: self contents.
	self availableForWriting > 0
		ifTrue: [ stream space; nextPut: $+; print: self availableForWriting ].
	stream nextPut: $)! !

!ZdcIOBuffer methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
reset
	"Do a hard reset"
	
	readPointer := 0.
	writePointer := 0! !

!ZdcIOBuffer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:46'!
size
	"Return the number of actual elements available"
	
	^ self availableForReading! !

Stream subclass: #ZdcAbstractSocketStream
	instanceVariableNames: 'socket readBuffer writeBuffer timeout debug'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcAbstractSocketStream commentStamp: 'SvenVanCaekenberghe 5/17/2011 18:56' prior: 0!
I am ZdcAbstractSocketStream, a binary read/write stream for socket communication.

Interally, IO is done through a read and a write ZdcIOBuffer.

I am abstract, my subclasses should implement actual IO through a delegate.!

!ZdcAbstractSocketStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
on: object
	^ (self basicNew)
		initialize;
		on: object;
		yourself! !

!ZdcAbstractSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
atEnd
	"I am atEnd when there is no more data to be read and there never will be.
	This means that readBuffer must be empty, 
	there must be no more unread data available at the socket,
	and the socket must be closed"
	
	^ self subclassResponsibility ! !

!ZdcAbstractSocketStream methodsFor: 'compatibility' stamp: 'SvenVanCaekenberghe 11/10/2011 13:06'!
autoFlush: boolean
	"I flush when asked to, or when my buffer is full."! !

!ZdcAbstractSocketStream methodsFor: 'compatibility' stamp: 'SvenVanCaekenberghe 11/10/2011 13:07'!
binary
	"I am always binary."! !

!ZdcAbstractSocketStream methodsFor: 'compatibility' stamp: 'SvenVanCaekenberghe 11/10/2011 13:08'!
bufferSize: numberOfBytes
	"Not yet implemented. See #initializeBuffers."! !

!ZdcAbstractSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/20/2011 13:27'!
close
	"Close the stream, flush if necessary"

	self flush.
	socket isNil ifTrue: [ ^ self ].
	self socketClose.  
	socket := nil! !

!ZdcAbstractSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
collectionSpecies
	"Fixed to ByteArray since we are binary"
	
	^ ByteArray
	! !

!ZdcAbstractSocketStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
debug: aBoolean
	debug := aBoolean ! !

!ZdcAbstractSocketStream methodsFor: 'private in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
fillReadBuffer
	"Ask the delegate to fill the read buffer with data. 
	Wait for at least some data."
	
	self subclassResponsibility  ! !

!ZdcAbstractSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
flush
	"Flush all pending output that might be in the write buffer"
	
	self flushWriteBuffer ! !

!ZdcAbstractSocketStream methodsFor: 'private out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
flushWriteBuffer
	"Ask the delegate to write all data in the write buffer. Fail if not successful"
	
	self subclassResponsibility ! !

!ZdcAbstractSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/18/2011 15:59'!
initialize 
	timeout := 5.
	debug := false.
	self initializeBuffers ! !

!ZdcAbstractSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/18/2011 15:59'!
initializeBuffers
	readBuffer := ZdcIOBuffer onByteArrayOfSize: 4096.
	writeBuffer := ZdcIOBuffer onByteArrayOfSize: 4096! !

!ZdcAbstractSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
isBinary
	"We are always binary"
	
	^ true! !

!ZdcAbstractSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
isConnected
	"Are we connected at the socket level ?"
	
	^ self subclassResponsibility ! !

!ZdcAbstractSocketStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
log: object
	debug ifTrue: [
		Transcript cr; show: object value ]
	! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next
	"Read and return a single byte"
	
	readBuffer isEmpty ifTrue: [ self fillReadBuffer ].
	^ readBuffer isEmpty
		ifTrue: [ nil ]
		ifFalse: [ readBuffer next ]! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: requestedCount 
	"Read requestedCount bytes and return them as a ByteArray.
	If less are available, a smaller ByteArray will be returned."

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self 
		next: requestedCount 
		into: collection 
		startingAt: 1   ! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| read |
	read := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ read = requestedCount 
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + read - 1 ]     ! !

!ZdcAbstractSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: count putAll: collection
	"Write count bytes from collection"
	
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZdcAbstractSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: count putAll: collection startingAt: offset
	"Write count bytes from collection starting at offset.
	This is an inefficient abstract implementation writing bytes one by one."
	
	0 to: count - 1 do: [ :each | 
		self nextPut: (collection at: offset + each) ]! !

!ZdcAbstractSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
nextPut: object
	"Write a single byte"
	
	writeBuffer isFull ifTrue: [ self flushWriteBuffer ].
	writeBuffer nextPut: object
	! !

!ZdcAbstractSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
nextPutAll: collection
	"Write a collection of bytes"
	
	self 
		next: collection size 
		putAll: collection 
		startingAt: 1! !

!ZdcAbstractSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
on: object
	socket := object! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/19/2011 12:50'!
peek
	"Peek and return a single byte"
	
	readBuffer isEmpty ifTrue: [ self fillReadBuffer ].
	^ readBuffer isEmpty
		ifTrue: [ nil ]
		ifFalse: [ readBuffer peek]! !

!ZdcAbstractSocketStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset,
	returning the number of elements read, there could be less elements available.
	This is an inefficient abstract implementation, reading bytes one by one."
	
	0 to: requestedCount - 1 do: [ :count | | object |
		(object := self next) ifNil: [ ^ count ].  
		collection at: offset + count put: object ].
	^ requestedCount! !

!ZdcAbstractSocketStream methodsFor: 'compatibility' stamp: 'SvenVanCaekenberghe 11/10/2011 13:07'!
shouldSignal: boolean
	"I always signal exceptional situations."! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
skip: count
	"Skip over count bytes.
	This is an inefficient abstract implementation skipping bytes one by one."
	
	count timesRepeat: [ self next ]! !

!ZdcAbstractSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
socket
	"Return the underlying socket object that I delegate I/O to"
	
	^ socket! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 12:53'!
socketClose
	socket close ! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 13:04'!
socketConnectTo: hostAddress port: portNumber
	socket 
		connectTo: hostAddress 
		port: portNumber 
		waitForConnectionFor: self timeout        ! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 12:58'!
socketIsConnected
	^ socket isConnected and: [ socket isOtherEndClosed not ]! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 12:59'!
socketIsDataAvailable
	^ socket dataAvailable ! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 12:56'!
socketReceiveDataInto: bytes startingAt: offset count: count
	^ socket 
		primSocket: socket socketHandle 
		receiveDataInto: bytes
		startingAt: offset
		count: count! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 13:13'!
socketSendData: bytes startingAt: offset count: count
	^ socket 
		sendSomeData: bytes 
		startIndex: offset 
		count: count ! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 13:03'!
socketWaitForData
	^ socket waitForDataFor: self timeout
	! !

!ZdcAbstractSocketStream methodsFor: 'private socket' stamp: 'SvenVanCaekenberghe 5/20/2011 13:03'!
socketWaitForSendDone
	^ socket waitForSendDoneFor: self timeout
	! !

!ZdcAbstractSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
timeout
	"Return the number of seconds we wait for socket IO operations"
	
	^ timeout ! !

!ZdcAbstractSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
timeout: seconds
	"Set the number of seconds we wait for socket IO operations"
	
	timeout := seconds! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
upTo: value 
	"Read bytes upto but not including value and return them as a ByteArray.
	If value is not found, return the entire contents of the stream.
	This is an inefficient abstract implementation reading bytes one by one."
	
	^ self collectionSpecies 
		streamContents: [ :writeStream | | element |
			[ self atEnd or: [ (element := self next) = value ] ] whileFalse: [ 
				writeStream nextPut: element ] ]! !

!ZdcAbstractSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
upToEnd
	"Read bytes until the stream is atEnd and return them as a ByteArray.
	This is an inefficient abstract implementation reading bytes one by one."

	^ self collectionSpecies 
		streamContents: [ :writeStream |
			[ self atEnd ] whileFalse: [ 
				writeStream nextPut: self next ] ]! !

ZdcAbstractSocketStream subclass: #ZdcSimpleSocketStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcSimpleSocketStream commentStamp: 'SvenVanCaekenberghe 5/17/2011 18:56' prior: 0!
I am ZdcSimpleSocketStream.

I am a ZdcAbstractSocketStream.

I do a minimal implementation of the requirements defined by my superclass.

I rely on the rather inefficient implementation of my superclass' methods, 
funneling all access through #next and #nextPut.!

ZdcSimpleSocketStream subclass: #ZdcOptimizedSocketStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcOptimizedSocketStream commentStamp: 'SvenVanCaekenberghe 5/17/2011 18:57' prior: 0!
I am ZnOptimizedSocketStream.

I am a ZdcSimpleSocketStream.

I re-implement the critical operations dealing with bulk input and output 
more efficiently to work with buffer sized chunks, bypassing #next and #nextPut:!

!ZdcOptimizedSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: count putAll: collection startingAt: offset
	"Write count bytes from collection starting at offset. Overwritten, optimized"
	
	| totalWritten |
	totalWritten := 0.
	[ | toWrite |
		toWrite := (count - totalWritten) min: writeBuffer availableForWriting.
		writeBuffer next: toWrite putAll: collection startingAt: offset + totalWritten.
		totalWritten := totalWritten + toWrite.
		totalWritten = count ] whileFalse: [ self flushWriteBuffer ]! !

!ZdcOptimizedSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset,
	returning the number of elements read. Overwritten, optimized"
	
	| totalRead |
	totalRead := 0.
	[ | read |
		(read := readBuffer availableForReading min: (requestedCount - totalRead)) > 0
			ifTrue: [  
				collection 
					replaceFrom: offset + totalRead to: offset + totalRead + read - 1 
					with: readBuffer buffer startingAt: readBuffer contentsStart.
				readBuffer advanceReadPointer: read.
				totalRead := totalRead + read ].
		totalRead = requestedCount or: [ self isConnected not ] ] whileFalse: [ self fillReadBuffer ].   
	^ totalRead! !

!ZdcOptimizedSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
skip: count
	"Skip over count bytes. Overwritten, optimized"
	
	| skipCount |
	skipCount := 0.
	[ | leftToSkip skipping |
		leftToSkip := count - skipCount.
		skipping := readBuffer availableForReading min: leftToSkip.
		readBuffer advanceReadPointer: skipping.
	     (skipCount := skipCount + skipping) = count ] whileFalse: [ self fillReadBuffer ]! !

!ZdcOptimizedSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/20/2011 13:35'!
upTo: value 
	"Read bytes upto but not including value and return them as a ByteArray.
	If value is not found, return the entire contents of the stream. Overwritten, optimized"
	
	^ self collectionSpecies 
		streamContents: [ :writeStream |
			[ | position |
				position := 0.
				readBuffer isEmpty ifFalse: [ | count |
					position := readBuffer buffer indexOf: value startingAt: readBuffer contentsStart.
					count := position = 0 ifTrue: [ readBuffer availableForReading ] ifFalse: [ position - readBuffer contentsStart ].
					writeStream next: count putAll: readBuffer buffer startingAt: readBuffer contentsStart.
					readBuffer advanceReadPointer: count.
					position = 0 ifFalse: [ readBuffer advanceReadPointer: 1 ] ].   
				position ~= 0 or: [ self atEnd ] ] whileFalse: [ 
					readBuffer isEmpty ifTrue: [ self fillReadBuffer ] ] ]! !

!ZdcOptimizedSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
upToEnd
	"Read bytes until the stream is atEnd and return them as a ByteArray. Overwritten, optimized"

	^ self collectionSpecies 
		streamContents: [ :writeStream |
			[	
				readBuffer isEmpty ifFalse: [
					writeStream 
						next: readBuffer availableForReading 
						putAll: readBuffer buffer 
						startingAt: readBuffer contentsStart.
					readBuffer advanceReadPointer: readBuffer availableForReading ].   
				self atEnd ] whileFalse: [ 
					readBuffer isEmpty ifTrue: [ self fillReadBuffer ] ] ]! !

ZdcOptimizedSocketStream subclass: #ZdcSecureSocketStream
	instanceVariableNames: 'sslSession in out connecting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcSecureSocketStream commentStamp: 'SvenVanCaekenberghe 5/18/2011 11:50' prior: 0!
I am ZdcSecureSocketStream, a binary read/write stream for SSL communication.

I am a ZdcOptimizedSocketStream.

When I am used as a client, call #connect on me before using me as a normal stream.

When I am used as a server, call #accept on me before using me as a normal stream.

Currently, certificate management is ignored.
!

!ZdcSecureSocketStream methodsFor: 'ssl' stamp: 'SvenVanCaekenberghe 12/13/2011 21:06'!
accept
	"Do the SSL server handshake."
	
	| count result |
	self resetEncryptedBuffers.
	count := 0.
	connecting := true.
	[ self sslSession isConnected ] whileFalse: [
		count := super fillBytes: in startingAt: 1 count: in size.
		result := self sslSession accept: in from: 1 to: count into: out.
		result < -1 ifTrue: [ 
			^ self sslException: 'accept failed' code: result ].
		result > 0 ifTrue: [ 
			super flushBytes: out startingAt: 1 count: result ] ].
	connecting := false! !

!ZdcSecureSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/18/2011 11:49'!
close
	"Close the stream, flush if necessary. 
	Destory the SSLSession object."

	super close.
	sslSession isNil ifTrue: [ ^ self ]. 
	sslSession destroy.
	sslSession := nil! !

!ZdcSecureSocketStream methodsFor: 'ssl' stamp: 'SvenVanCaekenberghe 9/21/2011 13:48'!
connect
	"Do the SSL client handshake."
	
	| count result |
	self resetEncryptedBuffers.
	count := 0.
	connecting := true.
	[ (result := self sslSession connect: in from: 1 to: count into: out) = 0 ] whileFalse: [
		result < -1 ifTrue: [ 
			^ self sslException: 'connect failed' code: result ].
		result > 0 ifTrue: [ 
			super flushBytes: out startingAt: 1 count: result ].
		count := super fillBytes: in startingAt: 1 count: in size ].
	connecting := false
	! !

!ZdcSecureSocketStream methodsFor: 'private in' stamp: 'SvenVanCaekenberghe 12/9/2011 10:06'!
fillBytes: bytes startingAt: offset count: count
	"Ask to read count elements into bytes starting at offset. Do not wait. Return read count.
	Overwritten: decrypt and if necessary ask the socket for encrypted data using a super call."
	
	| processedCount processData inCount |
	self isConnected ifFalse: [ ConnectionClosed signal: 'Cannot read data' ].  
	processData := [
		processedCount < 0 ifTrue: [ ^ self sslException: 'decrypt failed' code: processedCount ].
		processedCount > 0
			ifTrue: [
				bytes replaceFrom: offset to: offset + processedCount - 1 with: out startingAt: 1.
				^ processedCount ] ].
	self resetEncryptedBuffers.
	processedCount := self sslSession decrypt: in from: 1 to: 0 into: out.
	processData value.
	inCount := super fillBytes: in startingAt: 1 count: in size.
	processedCount := self sslSession decrypt: in from: 1 to: inCount into: out.
	processData value.
	^ 0! !

!ZdcSecureSocketStream methodsFor: 'private out' stamp: 'SvenVanCaekenberghe 12/9/2011 09:37'!
flushBytes: bytes startingAt: offset count: count
	"Ask to write count bytes starting from offset. Wait. Fail if not successful
	Overwritten: first encrypt the data, then send it to the socket using a super call."
	
	| remaining currentOffset |
	self isConnected ifFalse: [ ConnectionClosed signal: 'Cannot write data' ].  
	remaining := count.
	currentOffset := offset.
	[ remaining > 0 ] whileTrue: [ | chunkCount writeCount |
		self resetEncryptedBuffers.
		chunkCount := 4096 min: remaining.
		writeCount := self sslSession encrypt: bytes from: currentOffset to: currentOffset + chunkCount - 1 into: out.
		writeCount < 0 ifTrue: [ ^ self sslException: 'encrypt failed' code: writeCount ].
		super flushBytes: out startingAt: 1 count: writeCount.
		remaining := remaining - chunkCount.
		currentOffset := currentOffset + chunkCount ]! !

!ZdcSecureSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/18/2011 16:00'!
initialize 
	super initialize.
	connecting := false! !

!ZdcSecureSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/9/2011 12:47'!
initializeBuffers
	"The maximum payload message length of a TLS record is 16Kb,
	add a margin for the header and footer."
	
	readBuffer := ZdcIOBuffer onByteArrayOfSize: 16 * 1024.
	writeBuffer := ZdcIOBuffer onByteArrayOfSize: 4 * 1024.
	in := ByteArray new: 4096.
	out := ByteArray new: (16 + 1) * 1024
! !

!ZdcSecureSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/18/2011 11:49'!
isConnected
	"Are we connected at the socket level ?
	Has the SSL handshake been done successfully ?"

	^ super isConnected 
		and: [ sslSession notNil and: [ connecting or: [ sslSession isConnected ] ] ]! !

!ZdcSecureSocketStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/18/2011 11:49'!
resetEncryptedBuffers
	"Not necessary, but useful for debugging"
	
	in atAllPut: 0.
	out atAllPut: 0! !

!ZdcSecureSocketStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/18/2011 11:49'!
sslException: text code: code
	self error: 
		(String streamContents: [ :stream |
			stream << 'SSL Exception: ' << text << ' [code:'.
			stream print: code.
			stream << ']' ]) ! !

!ZdcSecureSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2011 09:57'!
sslSession
	"Return the underlying ZdcAbstractSSLSession object 
	that we use for encryption and decryption, 
	as well as connection setup handshaking, connect and accept"
	
	sslSession isNil ifTrue: [ sslSession := self sslSessionClass new ].
	^ sslSession! !

!ZdcSecureSocketStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2011 11:51'!
sslSessionClass
	^ ZdcPluginSSLSession ! !

ZdcOptimizedSocketStream subclass: #ZdcSocketStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zodiac-Core'!

!ZdcSocketStream commentStamp: 'SvenVanCaekenberghe 5/17/2011 18:57' prior: 0!
I am ZdcSocketStream.

I am a ZdcOptimizedSocketStream.

I further optimize my superclass' methods dealing with bulk input and output
to transparently bypass the internal buffers when this makes sense.!

!ZdcSocketStream methodsFor: 'stream out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
next: count putAll: collection startingAt: offset
	"Write count bytes from collection starting at offset. Overwritten, optimized"
	
	count <= (writeBuffer bufferSize / 8)  
		ifTrue: [
			count > writeBuffer availableForWriting ifTrue: [ self flushWriteBuffer ].  
			writeBuffer next: count putAll: collection startingAt: offset ]
		ifFalse: [
			self flushWriteBuffer.
			self flushBytes: collection startingAt: offset count: count ]! !

!ZdcSocketStream methodsFor: 'stream in' stamp: 'SvenVanCaekenberghe 5/20/2011 13:07'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset,
	returning the number of elements read. Overwritten, optimized"
	
	| totalRead read |
	totalRead := 0.
	(read := readBuffer availableForReading min: requestedCount) > 0
		ifTrue: [  
			collection replaceFrom: offset to: offset + read - 1 with: readBuffer buffer startingAt: readBuffer contentsStart.
			readBuffer advanceReadPointer: read.
			(totalRead := totalRead + read) = requestedCount ifTrue: [ ^ totalRead ] ].  
	[
		read := self fillBytes: collection startingAt: offset + totalRead count: requestedCount - totalRead.
		(totalRead := totalRead + read) = requestedCount or: [ self isConnected not ] ] whileFalse: [ 
			self socketWaitForData ].
	^ totalRead! !

!ZdcSimpleSocketStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
openConnectionToHost: hostIP port: portNumber
	| platformSocket |
	platformSocket := Socket newTCP.
	^ (self on: platformSocket)
		connectTo: hostIP port: portNumber;
		yourself! !

!ZdcSimpleSocketStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
openConnectionToHostNamed: hostName port: portNumber
	| platformSocket socketStream hostIP |
	platformSocket := Socket newTCP.
	socketStream := self on: platformSocket.
	hostIP := NetNameResolver addressForName: hostName timeout: socketStream timeout.
	^ socketStream 
		connectTo: hostIP port: portNumber;
		yourself! !

!ZdcSimpleSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/20/2011 13:07'!
atEnd
	"I am atEnd when there is no more data to be read and there never will be.
	This means that readBuffer must be empty, there must be no more unread data 
	available at the socket, and the socket must be closed"
	
	readBuffer isEmpty ifFalse: [ ^ false ].
	(socket notNil and: [ self socketIsDataAvailable ]) 
		ifTrue: [
			"Assuming there really is data available, read it and recurse"
			self fillReadBufferNoWait.
			self atEnd ].   
	^ self isConnected not! !

!ZdcSimpleSocketStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/20/2011 13:07'!
connectTo: hostAddress port: portNumber
	"Connect our socket to hostAddress:portNumber.
	Wait up to timeout for a connection"
	
	self socketConnectTo: hostAddress port: portNumber! !

!ZdcSimpleSocketStream methodsFor: 'private in' stamp: 'SvenVanCaekenberghe 5/20/2011 14:21'!
fillBytes: bytes startingAt: offset count: count
	"Ask the socket to read count elements into bytes starting at offset. Do not wait. Return read count."
	
	self isConnected ifFalse: [ ConnectionClosed signal: 'Cannot read data' ].  
	^ self 
		socketReceiveDataInto: bytes 
		startingAt: offset 
		count: count! !

!ZdcSimpleSocketStream methodsFor: 'private in' stamp: 'SvenVanCaekenberghe 5/20/2011 13:05'!
fillReadBuffer
	"Ask the socket to fill the read buffer with data. Wait for a data."
	
	self fillReadBufferNoWait.
	(readBuffer isEmpty and: [ self isConnected ])
		ifTrue: [ 
			self socketWaitForData. "when successful, recurse, else signal exception"
			self fillReadBuffer ]  ! !

!ZdcSimpleSocketStream methodsFor: 'private in' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
fillReadBufferNoWait
	"Ask the socket to fill the read buffer with data. Do not wait for data."
	
	| readCount |
	readBuffer compact.
	readCount := self 
		fillBytes: readBuffer buffer 
		startingAt: readBuffer freeSpaceStart
		count: readBuffer availableForWriting .
	readBuffer advanceWritePointer: readCount.
	^ readCount! !

!ZdcSimpleSocketStream methodsFor: 'private out' stamp: 'SvenVanCaekenberghe 5/20/2011 13:05'!
flushBytes: bytes startingAt: offset count: count
	"Ask the socket to write count bytes starting from offset. Wait. Fail if not successful"
	
	| writeCount |
	self isConnected ifFalse: [ ConnectionClosed signal: 'Cannot write data' ].  
	writeCount := 0.
	[ | written |
		written := self socketSendData: bytes startingAt: offset + writeCount count: count - writeCount.
		(writeCount := writeCount + written) = count ] whileFalse: [
		self socketWaitForSendDone
			ifFalse: [ ConnectionTimedOut signal: 'Data send timed out.' ] ]! !

!ZdcSimpleSocketStream methodsFor: 'private out' stamp: 'SvenVanCaekenberghe 5/17/2011 18:52'!
flushWriteBuffer
	"Ask the socket to write all data in the write buffer. Fail if not successful"
	
	writeBuffer isEmpty ifTrue: [ ^ self ].  
	self 
		flushBytes: writeBuffer buffer
		startingAt: writeBuffer contentsStart
		count: writeBuffer availableForReading. 
	writeBuffer reset ! !

!ZdcSimpleSocketStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/20/2011 13:02'!
isConnected
	"Are we connected at the socket level ?"
	
	^ socket notNil and: [ self socketIsConnected ]! !
